#!/usr/bin/env python3
import argparse


def main(args):
    """
    Main function for illustrating the use of argparse

    :param args: List of arguments to the main function.
    """
    parser = argparse.ArgumentParser()
    # Optional positional integer argument with a default value of 20
    parser.add_argument("number", nargs="?", type=int, default=20)
    # Optional file argument with a default path
    # "-f" is a shorthand to avoid having to type "--file"
    # If default is a string it is parsed using the specified `type` argument. Otherwise it is used as-is
    parser.add_argument("-f", "--file", type=argparse.FileType("r", encoding="utf-8"), default="path/to/some_file.txt")
    # Optional integer argument with a default value of 10
    parser.add_argument("-m", "--max-count", type=int, default=10)

    # Without arguments `parse_args` will use `sys.argv` directly which is how it's used in the code for assignment 2
    args = parser.parse_args(args)

    # `args` is of type `Namespace` which stores all arguments as attributes/instance variables
    # Note that the hyphen in --max-count is replaced by by an underscore here to make it a valid python identifier
    print("Number:", args.number)
    print("Max Count:", args.max_count)

    # A with statement can be used and optionally assigned to a variable
    # Could also be just `with args.file:` and then `for line in args.file:`
    with args.file as file:
        # Handle your files in some way
        for line in file:
            print(line, end="")


if __name__ == "__main__":
    # Template for calling a main function with arguments
    # sys is imported here since it's not used unless this module is run directly
    import sys

    # Passes all arguments except for the first element which is the name of the file that is run
    # and isn't handled by `argparse` when it's passed like this
    main(sys.argv[1:])
